@@ -36,10 +36,6 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
-#define PI                    3.14159265358979323846264338f
-#define sqrt2PI               2.50662827463100050241576528f
-#define IPI4                  0.75112554446f
-
 #include "fcwt.h"
 
 Morlet::Morlet(float bandwidth) {
@@ -75,14 +71,18 @@ void Morlet::generate(float* real, float* imag, int size, float scale) {
     width = getSupport(scale);
     float norm = (float)size * ifb * IPI4;
     
+    //cout << scale << " [";
     for(int t=0; t < width*2+1; t++) {
         tmp1 = (float)(t - width)/scale;
         tmp2 = exp(-(tmp1*tmp1)/(fb2));
         
         real[t] = norm*tmp2*cos(tmp1*2.0f*PI)/scale;
         imag[t] = norm*tmp2*sin(tmp1*2.0f*PI)/scale;
+        //cout << real[t]*real[t]+imag[t]*imag[t] << ",";
     }
+    //cout << "]" << endl;
 }
+
 void Morlet::getWavelet(float scale, complex<float>* pwav, int pn) {
     int w = getSupport(scale);
 
@@ -132,7 +132,7 @@ void Scales::getScales(float *pfreqs, int pnf) {
 
 void Scales::getFrequencies(float *pfreqs, int pnf) { 
     for(int i=0;i<pnf;i++) { 
-        pfreqs[i]=(((float)fs)/scales[i]); 
+        pfreqs[i]=((float)fs)/scales[i]; 
     }; 
 };
 
@@ -146,7 +146,7 @@ void Scales::calculate_logscale_array(float base, float four_wavl, int fs, float
     
     //Cannot pass the nyquist frequency
     assert(("Max frequency cannot be higher than the Nyquist frequency (fs/2)", f1 <= fs/2));
-    
+
     float power0 = log(s0)/log(base);
     float power1 = log(s1)/log(base);
     float dpower = power1-power0;
@@ -306,6 +306,7 @@ void FCWT::daughter_wavelet_multiplication(fftwf_complex *input, fftwf_complex *
     
     return;
 }
+
 void FCWT::create_FFT_optimization_plan(int maxsize, int flags) {
     
     int nt = find2power(maxsize);
@@ -419,7 +420,7 @@ void FCWT::fft_normalize(complex<float>* out, int size) {
     }
 }
 
-void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales) {
+void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales, bool complexinput) {
     
     fftwf_complex *Ihat, *O1;
     size = psize;
@@ -427,9 +428,6 @@ void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales
     //Find nearest power of 2
     const int nt = find2power(size);
     const int newsize = 1 << nt;
-    
-    //Initialize input and mother wavelet memory
-    float* input = (float*)calloc(newsize,sizeof(float));
 
     //Initialize intermediate result
     #ifdef _WIN32
@@ -439,9 +437,8 @@ void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales
         Ihat = (fftwf_complex*)aligned_alloc(32, newsize*sizeof(fftwf_complex));
         O1 = (fftwf_complex*)aligned_alloc(32, newsize*sizeof(fftwf_complex));
     #endif
-    
+
     //Copy input to new input buffer
-    memcpy(input,pinput,sizeof(float)*size);
     memset(Ihat,0,sizeof(fftwf_complex)*newsize);
     memset(O1,0,sizeof(fftwf_complex)*newsize);
     
@@ -459,9 +456,20 @@ void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales
     load_FFT_optimization_plan();
 
     // //Perform forward FFT on input signal
-    p = fftwf_plan_dft_r2c_1d(newsize, &input[0], Ihat, FFTW_ESTIMATE);
+    float *input;
+    if(complexinput) {
+        input = (float*)calloc(newsize,sizeof(complex<float>));
+        memcpy(input,pinput,sizeof(complex<float>)*size);
+        p = fftwf_plan_dft_1d(newsize, (fftwf_complex*)input, Ihat, FFTW_FORWARD, FFTW_ESTIMATE);
+    } else {
+        input = (float*)calloc(newsize,sizeof(float));
+        memcpy(input,pinput,sizeof(float)*size);
+        p = fftwf_plan_dft_r2c_1d(newsize, &input[0], Ihat, FFTW_ESTIMATE);
+    }
+
     fftwf_execute(p);
     fftwf_destroy_plan(p);
+    free(input);
     
     pinv = fftwf_plan_dft_1d(newsize, O1, (fftwf_complex*)poutput, FFTW_BACKWARD, FFTW_ESTIMATE);
     
@@ -483,19 +491,22 @@ void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales
     
     //Cleanup
     fftwf_destroy_plan(pinv);
-    fftwf_free(Ihat);
-    fftwf_free(O1);
-    free(input);
+    #ifdef _WIN32
+        _aligned_free(Ihat);
+        _aligned_free(O1);
+    #else
+        free(Ihat);
+        free(O1);
+    #endif
 }
 
-void FCWT::cwt(float *pinput, int psize, Scales *scales, complex<float>** poutput, int* pnoutput) {
-    *poutput = (complex<float>*)malloc(sizeof(complex<float>)*psize*scales->nscales);
-    *pnoutput = psize*scales->nscales;
-    cwt(pinput,psize,*poutput,scales);
+
+void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales) {
+    cwt(pinput,psize,poutput,scales,false);
 }
 
-void FCWT::cwt(float *pinput, int psize, Scales *scales, complex<float>* poutput, int pnoutput) {
-    cwt(pinput,psize,poutput,scales);
+void FCWT::cwt(complex<float> *pinput, int psize, complex<float>* poutput, Scales *scales) {
+    cwt((float*)pinput,psize,poutput,scales,true);
 }
 
 void FCWT::cwt(float *pinput, int psize, Scales *scales, complex<float>* poutput, int pn1, int pn2) {
@@ -503,11 +514,8 @@ void FCWT::cwt(float *pinput, int psize, Scales *scales, complex<float>* poutput
     cwt(pinput,psize,poutput,scales);
 }
 
-void FCWT::cwt(float *pinput, int psize, Scales *scales, complex<float>* poutput, int pn1, int pn2, float *pfreqs, int pnf) {
+void FCWT::cwt(complex<float> *pinput, int psize, Scales *scales, complex<float>* poutput, int pn1, int pn2) {
     assert((psize*scales->nscales) == (pn1*pn2));
-    assert(false);
     cwt(pinput,psize,poutput,scales);
-    for(int i=0; i<pnf;i++) {
-        pfreqs[i] = scales->scales[i];
-    }
 }
+
